{ ---------------------------------------- }

Program SysMem;

{ Simple CP/M memory mapper }
{ Martin Hepperle, 11/2018 }

Var
  HI, LO:     Integer;
  CCPAddr:    Real;
  BDOSAddr:   Real;
  BIOSAddr:   Real;
  BIOSLength: Real;
  BDOSLength: Real;
  I: Integer;
Type
  String2 = String[2];
  String4 = String[4];

Const
  CCPLength = $800;

{ ---------------------------------------- }

Function Hex2( Value: Byte ): String2;
{ Convert a byte to a string of two hex characters [0-9,A-F]  }
Var
  Nibble: Byte;
  Buffer: String2;
Begin
  Buffer := '  ';
  Nibble := (Value and $00F0) shr 4;
  If Nibble < 10 Then Buffer[1] := Chr(Nibble+48) Else Buffer[1] := Chr(Nibble+55);
  Nibble := Value and $000F;
  If Nibble < 10 Then Buffer[2] := Chr(Nibble+48) Else Buffer[2] := Chr(Nibble+55);
  Hex2 := Buffer;
End;

{ ---------------------------------------- }

Function Hex4( Addr: Real ): String4;
{ Convert a integer to a string of four hex characters [0-9,A-F]  }
Var
  HI, LO: Byte;
  Buffer: String4;
Begin
  Buffer := '    ';
  HI := Trunc(Addr / 256.0);
  LO := Trunc(Addr - HI*256.0);
  Buffer := Concat ( Hex2(HI), Hex2(LO) );
  Hex4 := Buffer;
End;

{ ---------------------------------------- }

Begin

   { address of serial number - 6 }
   BDOSAddr := Mem[7]*256.0 + Mem[6] - 6.0;
   { cold start address }
   BIOSAddr := Mem[2]*256.0 + Mem[1] - 3.0;
   CCPAddr := Int(BDOSAddr - CCPLength);
   BDOSLength := BIOSAddr - BDOSAddr;
   BIOSLength := 62.0*1024.0  - BIOSAddr;


   WriteLn ( '*** CP/M Load Map ***' );

   Write ('BIOS begins at ' );
   Write ( Hex4(BIOSAddr), 'h =  (length = ', Hex4(BIOSLength), 'h)' );
   WriteLn;

   Write ('BDOS begins at ' );
   Write ( Hex4(BDOSAddr), 'h =  (length = ', Hex4(BDOSLength), 'h)' );
   WriteLn;

   Write ('CCP  begins at ' );
   Write ( Hex4(CCPAddr), 'h =  (length = ', Hex4(CCPLength), 'h)' );
   WriteLn;

End.
