      PROGRAM TEKTEST
C ---------------------------------------------
C
C     Test Tektronix output via FORTRAN
C     For use with Teraterm or xterm.
C
C     Martin Hepperle, 11/2018
C
C     f80 =iotest
C     l80 iotest/N,iotest/e
C
C ---------------------------------------------
      REAL PI, X, Y

      PI = 4.0*ATAN(1.0)

C     Microsoft FORTRAN 3.44 for CP/M
C     Units 1,3,5 are preassigned to TTY:
C     Unit 2 is preassigned to LST:
C     Unit 4 is preassigned to RDR: and PUN:


C     select output device
      CALL TINIT(1)

C     clear screen
      CALL TCLEAR

C     setup scaling
      CALL TSCAL(-0.1,6.9, -1.1,1.1)

C     plot x-axis
      CALL TMOVE(0.0,0.0)
      CALL TDRAW(2.0*PI,0.0)
C     x-ticks
      X = 0.0
150   CALL TMOVE(X,0.0)
      CALL TDRAW(X,-0.05)
      X = X + PI/10.0
      IF ( X .LE. 2.001*PI ) GOTO 150

C     plot y-axis
      CALL TMOVE(0.0,-1.0)
      CALL TDRAW(0.0, 1.0)
C     y-ticks
      Y = -1.0
160   CALL TMOVE(0.0, Y)
      CALL TDRAW(0.1,Y)
      Y = Y + 0.1
      IF ( Y .LE. 1.001 ) GOTO 160

C     draw sine curve
      X = 0.0
      Y = SIN(X)
      CALL TMOVE(X,Y)
200   Y = SIN(X)
      CALL TDRAW(X,Y)
      X = X + PI/20.0
      IF ( X .LE. 2.001*PI ) GOTO 200

C     draw cosine curve
      X = 0.0
      Y = COS(X)
      CALL TMOVE(X,Y)
300   Y = COS(X)
      CALL TDRAW(X,Y)
      X = X + PI/20.0
      IF ( X .LE. 2.001*PI ) GOTO 300

C     switch back to VT window
      CALL TCLOSE

      END

C ---------------------------------------------
C ---------------------------------------------
      SUBROUTINE TINIT(NDEV)
C
C     Assign output unit for Tektronix 4014
C
      LOGICAL*1 SEQ(5)
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN
C     ESC [ P mode h   Teraterm: switch to TEK Window
      DATA SEQ /27,'[','?',38,'h'/

      IDEV = NDEV

      WRITE(IDEV) SEQ

      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TCLEAR
C
C     Clear Tektronix 4014
C
      LOGICAL*1 SEQ(3)
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN
C     US ESC FF
      DATA SEQ /31, 27,12/

      WRITE(IDEV) SEQ

      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TCLOSE
C
C     Return to Alpha Mode of Tektronix 4014
C
      LOGICAL*1 SEQ(4)
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN
C     US     ESC [ l = RESET VT100
      DATA SEQ /31,27,'[','l'/

      WRITE(IDEV) SEQ

      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TSCAL(XLO,XHI,YLO,YHI)
C
C     Define scaling for Tektronix 4014
C
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN

      XMIN = XLO
      XMAX = XHI
      YMIN = YLO
      YMAX = YHI

      SCALX = 4096.0/(XMAX-XMIN)
      SCALY = 3120.0/(YMAX-YMIN)

      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TMOVE(X,Y)
C
C     Move to scaled point on Tektronix 4014
C
      REAL X,Y
      INTEGER IX,IY
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN

      IX = INT((X-XMIN)*SCALX)
      IY = INT((Y-YMIN)*SCALY)

      CALL TIMOVE(IX,IY)

      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TDRAW(X,Y)
C
C     Draw to scaled point on Tektronix 4014
C
      REAL X,Y
      INTEGER IX,IY
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN

      IX = INT((X-XMIN)*SCALX)
      IY = INT((Y-YMIN)*SCALY)

      CALL TIDRAW(IX,IY)

      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TIMOVE(IX,IY)
C
C     Move to device point on Tektronix 4014
C
      LOGICAL*1 SEQ
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN
C     GS
      DATA SEQ /29/

      WRITE(IDEV) SEQ

      CALL TIDRAW(IX,IY)


      RETURN
      END
C ---------------------------------------------
      SUBROUTINE TIDRAW(IX,IY)
C
C     Draw to device point on Tektronix 4014
C
      LOGICAL*1 BUFFER(5)
      COMMON /SCALE/IDEV,SCALX,SCALY,XMIN,YMIN

      BUFFER(1) = MOD(IY/128,32) + 32
      BUFFER(2) = MOD(IY,4)*4 + MOD(IX,4)+96
      BUFFER(3) = MOD(IY/4,32) + 96
      BUFFER(4) = MOD(IX/128,32) + 32
      BUFFER(5) = MOD(IX/4,32) + 64

      WRITE(IDEV) BUFFER

      RETURN
      END
C ---------------------------------------------
C ---------------------------------------------
